@@ -1,10 +1,13 @@
 import asyncio
 import datetime
+import os
+import tempfile
 import unittest
 from http.cookies import SimpleCookie
 from unittest import mock
 
 import pytest
+from yarl import URL
 
 from aiohttp import CookieJar
 
@@ -140,6 +143,25 @@ def test_constructor(loop, cookies_to_send, cookies_to_receive):
     assert jar._loop is loop
 
 
+def test_save_load(loop, cookies_to_send, cookies_to_receive):
+    file_path = tempfile.mkdtemp() + '/aiohttp.test.cookie'
+
+    # export cookie jar
+    jar_save = CookieJar(loop=loop)
+    jar_save.update_cookies(cookies_to_receive)
+    jar_save.save(file_path=file_path)
+
+    jar_load = CookieJar(loop=loop)
+    jar_load.load(file_path=file_path)
+
+    jar_test = SimpleCookie()
+    for cookie in jar_load:
+        jar_test[cookie.key] = cookie
+
+    os.unlink(file_path)
+    assert jar_test == cookies_to_receive
+
+
 def test_ctor_ith_default_loop(loop):
     asyncio.set_event_loop(loop)
     jar = CookieJar()
@@ -172,7 +194,7 @@ def test_domain_filter_ip_cookie_send(loop):
     )
 
     jar.update_cookies(cookies)
-    cookies_sent = jar.filter_cookies("http://1.2.3.4/").output(
+    cookies_sent = jar.filter_cookies(URL("http://1.2.3.4/")).output(
         header='Cookie:')
     assert cookies_sent == 'Cookie: shared-cookie=first'
 
@@ -180,7 +202,7 @@ def test_domain_filter_ip_cookie_send(loop):
 def test_domain_filter_ip_cookie_receive(loop, cookies_to_receive):
     jar = CookieJar(loop=loop)
 
-    jar.update_cookies(cookies_to_receive, "http://1.2.3.4/")
+    jar.update_cookies(cookies_to_receive, URL("http://1.2.3.4/"))
     assert len(jar) == 0
 
 
@@ -190,7 +212,7 @@ def test_preserving_ip_domain_cookies(loop):
         "shared-cookie=first; "
         "ip-cookie=second; Domain=127.0.0.1;"
     ))
-    cookies_sent = jar.filter_cookies("http://127.0.0.1/").output(
+    cookies_sent = jar.filter_cookies(URL("http://127.0.0.1/")).output(
         header='Cookie:')
     assert cookies_sent == ('Cookie: ip-cookie=second\r\n'
                             'Cookie: shared-cookie=first')
@@ -199,10 +221,10 @@ def test_preserving_ip_domain_cookies(loop):
 def test_ignore_domain_ending_with_dot(loop):
     jar = CookieJar(loop=loop, unsafe=True)
     jar.update_cookies(SimpleCookie("cookie=val; Domain=example.com.;"),
-                       "http://www.example.com")
-    cookies_sent = jar.filter_cookies("http://www.example.com/")
+                       URL("http://www.example.com"))
+    cookies_sent = jar.filter_cookies(URL("http://www.example.com/"))
     assert cookies_sent.output(header='Cookie:') == "Cookie: cookie=val"
-    cookies_sent = jar.filter_cookies("http://example.com/")
+    cookies_sent = jar.filter_cookies(URL("http://example.com/"))
     assert cookies_sent.output(header='Cookie:') == ""
 
 
@@ -220,11 +242,11 @@ class TestCookieJarBase(unittest.TestCase):
 
     def request_reply_with_same_url(self, url):
         self.jar.update_cookies(self.cookies_to_send)
-        cookies_sent = self.jar.filter_cookies(url)
+        cookies_sent = self.jar.filter_cookies(URL(url))
 
         self.jar.clear()
 
-        self.jar.update_cookies(self.cookies_to_receive, url)
+        self.jar.update_cookies(self.cookies_to_receive, URL(url))
         cookies_received = SimpleCookie()
         for cookie in self.jar:
             dict.__setitem__(cookies_received, cookie.key, cookie)
@@ -281,7 +303,7 @@ class TestCookieJarSafe(TestCookieJarBase):
             self.jar.update_cookies(self.cookies_to_send)
 
         with mock.patch.object(self.loop, 'time', return_value=send_time):
-            cookies_sent = self.jar.filter_cookies(url)
+            cookies_sent = self.jar.filter_cookies(URL(url))
 
         self.jar.clear()
 
@@ -352,12 +374,13 @@ class TestCookieJarSafe(TestCookieJarBase):
         })
 
     def test_domain_filter_host_only(self):
-        self.jar.update_cookies(self.cookies_to_receive, "http://example.com/")
+        self.jar.update_cookies(self.cookies_to_receive,
+                                URL("http://example.com/"))
 
-        cookies_sent = self.jar.filter_cookies("http://example.com/")
+        cookies_sent = self.jar.filter_cookies(URL("http://example.com/"))
         self.assertIn("unconstrained-cookie", set(cookies_sent.keys()))
 
-        cookies_sent = self.jar.filter_cookies("http://different.org/")
+        cookies_sent = self.jar.filter_cookies(URL("http://different.org/"))
         self.assertNotIn("unconstrained-cookie", set(cookies_sent.keys()))
 
     def test_secure_filter(self):
